import { Component } from '@angular/core';
import {
  CdkDragDrop,
  CdkDrag,
  CdkDropList,
  CdkDropListGroup,
  moveItemInArray,
  transferArrayItem,
} from '@angular/cdk/drag-drop';
import { Spell } from 'src/interfaces/spell';
import { DataService } from 'src/services/data/data.service';
import { ModalService } from 'src/services/modal/modal.service';
import { SpellsService } from 'src/services/spells/spells.service';
import { SpellModalComponent } from 'src/app/journal/spell-modal/spell-modal.component';
import { NgbTypeaheadModule } from '@ng-bootstrap/ng-bootstrap';
import { Observable, OperatorFunction } from 'rxjs';
import { debounceTime, distinctUntilChanged, map } from 'rxjs/operators';
import { FormsModule } from '@angular/forms';
import { JsonPipe } from '@angular/common';
import { FormControl } from '@angular/forms';
import { FullSpellcardComponent } from 'src/app/shared-components/full-spellcard/full-spellcard.component';

@Component({
  selector: 'app-journal-spellcards',
  templateUrl: './journal-spellcards.component.html',
  styleUrls: ['./journal-spellcards.component.scss'],
})
export class JournalSpellcardsComponent {
  public level0: Spell[] = [];
  public level1: Spell[] = [];
  public level2: Spell[] = [];
  public level3: Spell[] = [];
  public level4: Spell[] = [];
  public level5: Spell[] = [];
  public level6: Spell[] = [];
  public level7: Spell[] = [];
  public level8: Spell[] = [];
  public level9: Spell[] = [];

  public showSpellList: boolean[] = [
    true,
    true,
    true,
    true,
    true,
    true,
    true,
    true,
    true,
    true,
  ];

  public draggingIndex: number | undefined;

  public constructor(
    public dataAccessor: DataService,
    private modalAccessor: ModalService
  ) {
    this.loadSpells();
    this.hideEmptySpelllists();
  }

  ///////// FUNCTIONS //////////

  public showFullSpellcard(
    spell: Spell,
    level: number,
    spellIndex: number
  ): void {
    this.modalAccessor.openModal(FullSpellcardComponent, { spell: spell });
    const resultSubscription = this.modalAccessor.result$.subscribe(
      (result) => {
        resultSubscription.unsubscribe();
        if (result.state === 'delete') {
          this.getSpellList(level).splice(spellIndex, 1);
        } else if (result.state === 'update') {
          console.log('update');
          setTimeout(() => {
            this.openSpellModal(true, level, spellIndex);
          }, 100);
        } else if (result.state !== 'cancel') {
          console.log(result.state);
          throw new Error('Unexpected result state, please send a bug report.');
        }
      }
    );
  }

  public openSpellModal(
    isUpdate: boolean,
    level: number,
    spellIndex?: number
  ): void {
    this.modalAccessor.openModal(SpellModalComponent, {
      spell:
        spellIndex !== undefined
          ? JSON.parse(JSON.stringify(this.getSpellList(level)![spellIndex]))
          : undefined,
      isUpdate: isUpdate,
      level: level,
    });
    const resultSubscription = this.modalAccessor.result$.subscribe(
      (result) => {
        if (result.state === 'update') {
          // level was not modified
          if (level === result.data.level) {
            this.updateSpell(result.data, level!, spellIndex!);
          } else {
            // level was modified
            this.getSpellList(level).splice(spellIndex!, 1);
            this.addSpell(result.data, result.data.level);
          }
        } else if (result.state === 'add') {
          this.addSpell(result.data, level);
        }
        resultSubscription.unsubscribe();
      }
    );
  }

  public openSpellModalModifyOfficial(spell: Spell): void {
    this.modalAccessor.openModal(SpellModalComponent, {
      spell: spell,
      isUpdate: true,
      level: spell.level,
    });
    const resultSubscription = this.modalAccessor.result$.subscribe(
      (result) => {
        if (result.state === 'update') {
          this.addSpell(result.data, result.data.level);
        } else {
          throw new Error('Unexpected result state, please send a bug report.');
        }
        resultSubscription.unsubscribe();
      }
    );
  }

  public handleSpellSelection(event: any, level: number): void {
    const newSpell = event.spell;
    const isToModify = event.isToModify;

    if (isToModify) {
      this.openSpellModalModifyOfficial(newSpell);
    } else {
      this.addSpell(newSpell, level);
    }
  }

  public addSpell(spell: Spell, level: number) {
    this.getSpellList(level).push(spell);
    // this.updateSpellsInDatabase();
  }

  public updateSpell(spell: Spell, level: number, index: number): void {
    this.getSpellList(level)![index] = spell;
    this.updateSpellsInDatabase();
  }

  public getSpellList(level: number): Spell[] {
    switch (level) {
      case 0:
        return this.level0;
      case 1:
        return this.level1;
      case 2:
        return this.level2;
      case 3:
        return this.level3;
      case 4:
        return this.level4;
      case 5:
        return this.level5;
      case 6:
        return this.level6;
      case 7:
        return this.level7;
      case 8:
        return this.level8;
      case 9:
        return this.level9;
      default:
        throw new Error('Invalid spell level');
    }
  }

  public getSpellLevel(level: number): string {
    switch (level) {
      case 0:
        return 'Zaubertricks';
      case 1:
        return 'Level 1';
      case 2:
        return 'Level 2';
      case 3:
        return 'Level 3';
      case 4:
        return 'Level 4';
      case 5:
        return 'Level 5';
      case 6:
        return 'Level 6';
      case 7:
        return 'Level 7';
      case 8:
        return 'Level 8';
      case 9:
        return 'Level 9';
      default:
        return '';
    }
  }

  public toggleSpellList(index: number) {
    this.showSpellList[index] = !this.showSpellList[index];
  }

  public drop(event: CdkDragDrop<any[]>) {
    // console.log(event);
    if (event.previousContainer === event.container) {
      moveItemInArray(
        event.container.data,
        event.previousIndex,
        event.currentIndex
      );
    } else {
      transferArrayItem(
        event.previousContainer.data,
        event.container.data,
        event.previousIndex,
        event.currentIndex
      );

      // Update level of moved spell
      const movedSpell = event.container.data[event.currentIndex];
      const newContainer = event.container.id;

      switch (newContainer) {
        case 'cdk-drop-list-0':
          movedSpell.level = 0;
          break;
        case 'cdk-drop-list-1':
          movedSpell.level = 1;
          break;
        case 'cdk-drop-list-2':
          movedSpell.level = 2;
          break;
        case 'cdk-drop-list-3':
          movedSpell.level = 3;
          break;
        case 'cdk-drop-list-4':
          movedSpell.level = 4;
          break;
        case 'cdk-drop-list-5':
          movedSpell.level = 5;
          break;
        case 'cdk-drop-list-6':
          movedSpell.level = 6;
          break;
        case 'cdk-drop-list-7':
          movedSpell.level = 7;
          break;
        case 'cdk-drop-list-8':
          movedSpell.level = 8;
          break;
        case 'cdk-drop-list-9':
          movedSpell.level = 9;
          break;
      }
    }
  }

  public dragStart(index: number) {
    this.draggingIndex = index;
  }

  public dragEnd(event: any) {
    if (event.event.target.classList.contains('deletion-card')) {
      this.getSpellList(this.draggingIndex!).splice(
        event.source.element.nativeElement.id,
        1
      );
    }
    this.draggingIndex = undefined;
  }

  private loadSpells(): void {
    this.level0 = this.dataAccessor.spellLevel0;
    this.level1 = this.dataAccessor.spellLevel1;
    this.level2 = this.dataAccessor.spellLevel2;
    // this.level3 = this.dataAccessor.spellLevel3;
    // this.level4 = this.dataAccessor.spellLevel4;
    // this.level5 = this.dataAccessor.spellLevel5;
    // this.level6 = this.dataAccessor.spellLevel6;
    // this.level7 = this.dataAccessor.spellLevel7;
    // this.level8 = this.dataAccessor.spellLevel8;
    // this.level9 = this.dataAccessor.spellLevel9;
  }

  private hideEmptySpelllists(): void {
    this.showSpellList = [
      this.level0.length > 0,
      this.level1.length > 0,
      this.level2.length > 0,
      this.level3.length > 0,
      this.level4.length > 0,
      this.level5.length > 0,
      this.level6.length > 0,
      this.level7.length > 0,
      this.level8.length > 0,
      this.level9.length > 0,
    ];
  }

  private updateSpellsInDatabase(): void {
    this.dataAccessor.spellLevel0 = this.level0;
    this.dataAccessor.spellLevel1 = this.level1;
    this.dataAccessor.spellLevel2 = this.level2;
    // this.dataAccessor.spellLevel3 = this.level3;
    // this.dataAccessor.spellLevel4 = this.level4;
    // this.dataAccessor.spellLevel5 = this.level5;
    // this.dataAccessor.spellLevel6 = this.level6;
    // this.dataAccessor.spellLevel7 = this.level7;
    // this.dataAccessor.spellLevel8 = this.level8;
    // this.dataAccessor.spellLevel9 = this.level9;
  }
}
